@@ -1,7 +1,7 @@
 /*
  * conversion functions between pg_wchar and multibyte streams.
  * Tatsuo Ishii
- * $PostgreSQL: pgsql/src/backend/utils/mb/wchar.c,v 1.47.2.6 2007/03/26 11:43:09 ishii Exp $
+ * $PostgreSQL: pgsql/src/backend/utils/mb/wchar.c,v 1.58 2006/10/04 00:30:02 momjian Exp $
  *
  * WIN1250 client encoding updated by Pavel Behal
  *
@@ -19,16 +19,28 @@
 
 /*
  * conversion to pg_wchar is done by "table driven."
- * to add an encoding support, define mb2wchar_with_len(), mblen()
+ * to add an encoding support, define mb2wchar_with_len(), mblen(), dsplen()
  * for the particular encoding. Note that if the encoding is only
  * supported in the client, you don't need to define
  * mb2wchar_with_len() function (SJIS is the case).
+ *
+ * These functions generally assume that their input is validly formed.
+ * The "verifier" functions, further down in the file, have to be more
+ * paranoid.  We expect that mblen() does not need to examine more than
+ * the first byte of the character to discover the correct length.
+ *
+ * Note: for the display output of psql to work properly, the return values
+ * of the dsplen functions must conform to the Unicode standard. In particular
+ * the NUL character is zero width and control characters are generally
+ * width -1. It is recommended that non-ASCII encodings refer their ASCII
+ * subset to the ASCII routines to ensure consistency.
  */
 
 /*
  * SQL/ASCII
  */
-static int	pg_ascii2wchar_with_len
+static int
+			pg_ascii2wchar_with_len
 			(const unsigned char *from, pg_wchar *to, int len)
 {
 	int			cnt = 0;
@@ -40,19 +52,24 @@ static int	pg_ascii2wchar_with_len
 		cnt++;
 	}
 	*to = 0;
-	return (cnt);
+	return cnt;
 }
 
 static int
 pg_ascii_mblen(const unsigned char *s)
 {
-	return (1);
+	return 1;
 }
 
 static int
 pg_ascii_dsplen(const unsigned char *s)
 {
-	return (1);
+	if (*s == '\0')
+		return 0;
+	if (*s < 0x20 || *s == 0x7f)
+		return -1;
+
+	return 1;
 }
 
 /*
@@ -65,7 +82,8 @@ static int	pg_euc2wchar_with_len
 
 	while (len > 0 && *from)
 	{
-		if (*from == SS2 && len >= 2)	/* JIS X 0201 (so called "1 byte KANA") */
+		if (*from == SS2 && len >= 2)	/* JIS X 0201 (so called "1 byte
+										 * KANA") */
 		{
 			from++;
 			*to = (SS2 << 8) | *from++;
@@ -78,13 +96,14 @@ static int	pg_euc2wchar_with_len
 			*to |= *from++;
 			len -= 3;
 		}
-		else if ((*from & 0x80) && len >= 2)	/* JIS X 0208 KANJI */
+		else if (IS_HIGHBIT_SET(*from) && len >= 2)		/* JIS X 0208 KANJI */
 		{
 			*to = *from++ << 8;
 			*to |= *from++;
 			len -= 2;
 		}
-		else	/* must be ASCII */
+		else
+			/* must be ASCII */
 		{
 			*to = *from++;
 			len--;
@@ -93,7 +112,7 @@ static int	pg_euc2wchar_with_len
 		cnt++;
 	}
 	*to = 0;
-	return (cnt);
+	return cnt;
 }
 
 static inline int
@@ -105,11 +124,11 @@ pg_euc_mblen(const unsigned char *s)
 		len = 2;
 	else if (*s == SS3)
 		len = 3;
-	else if (*s & 0x80)
+	else if (IS_HIGHBIT_SET(*s))
 		len = 2;
 	else
 		len = 1;
-	return (len);
+	return len;
 }
 
 static inline int
@@ -121,11 +140,11 @@ pg_euc_dsplen(const unsigned char *s)
 		len = 2;
 	else if (*s == SS3)
 		len = 2;
-	else if (*s & 0x80)
+	else if (IS_HIGHBIT_SET(*s))
 		len = 2;
 	else
-		len = 1;
-	return (len);
+		len = pg_ascii_dsplen(s);
+	return len;
 }
 
 /*
@@ -134,13 +153,13 @@ pg_euc_dsplen(const unsigned char *s)
 static int	pg_eucjp2wchar_with_len
 			(const unsigned char *from, pg_wchar *to, int len)
 {
-	return (pg_euc2wchar_with_len(from, to, len));
+	return pg_euc2wchar_with_len(from, to, len);
 }
 
 static int
 pg_eucjp_mblen(const unsigned char *s)
 {
-	return (pg_euc_mblen(s));
+	return pg_euc_mblen(s);
 }
 
 static int
@@ -152,11 +171,11 @@ pg_eucjp_dsplen(const unsigned char *s)
 		len = 1;
 	else if (*s == SS3)
 		len = 2;
-	else if (*s & 0x80)
+	else if (IS_HIGHBIT_SET(*s))
 		len = 2;
 	else
-		len = 1;
-	return (len);
+		len = pg_ascii_dsplen(s);
+	return len;
 }
 
 /*
@@ -165,19 +184,19 @@ pg_eucjp_dsplen(const unsigned char *s)
 static int	pg_euckr2wchar_with_len
 			(const unsigned char *from, pg_wchar *to, int len)
 {
-	return (pg_euc2wchar_with_len(from, to, len));
+	return pg_euc2wchar_with_len(from, to, len);
 }
 
 static int
 pg_euckr_mblen(const unsigned char *s)
 {
-	return (pg_euc_mblen(s));
+	return pg_euc_mblen(s);
 }
 
 static int
 pg_euckr_dsplen(const unsigned char *s)
 {
-	return (pg_euc_dsplen(s));
+	return pg_euc_dsplen(s);
 }
 
 /*
@@ -205,7 +224,7 @@ static int	pg_euccn2wchar_with_len
 			*to |= *from++;
 			len -= 3;
 		}
-		else if ((*from & 0x80) && len >= 2)	/* code set 1 */
+		else if (IS_HIGHBIT_SET(*from) && len >= 2)		/* code set 1 */
 		{
 			*to = *from++ << 8;
 			*to |= *from++;
@@ -220,7 +239,7 @@ static int	pg_euccn2wchar_with_len
 		cnt++;
 	}
 	*to = 0;
-	return (cnt);
+	return cnt;
 }
 
 static int
@@ -228,11 +247,11 @@ pg_euccn_mblen(const unsigned char *s)
 {
 	int			len;
 
-	if (*s & 0x80)
+	if (IS_HIGHBIT_SET(*s))
 		len = 2;
 	else
 		len = 1;
-	return (len);
+	return len;
 }
 
 static int
@@ -240,11 +259,11 @@ pg_euccn_dsplen(const unsigned char *s)
 {
 	int			len;
 
-	if (*s & 0x80)
+	if (IS_HIGHBIT_SET(*s))
 		len = 2;
 	else
-		len = 1;
-	return (len);
+		len = pg_ascii_dsplen(s);
+	return len;
 }
 
 /*
@@ -261,7 +280,7 @@ static int	pg_euctw2wchar_with_len
 		if (*from == SS2 && len >= 4)	/* code set 2 */
 		{
 			from++;
-			*to = (SS2 << 24) | (*from++ << 16) ;
+			*to = (SS2 << 24) | (*from++ << 16);
 			*to |= *from++ << 8;
 			*to |= *from++;
 			len -= 4;
@@ -273,7 +292,7 @@ static int	pg_euctw2wchar_with_len
 			*to |= *from++;
 			len -= 3;
 		}
-		else if ((*from & 0x80) && len >= 2)	/* code set 2 */
+		else if (IS_HIGHBIT_SET(*from) && len >= 2)		/* code set 2 */
 		{
 			*to = *from++ << 8;
 			*to |= *from++;
@@ -288,7 +307,7 @@ static int	pg_euctw2wchar_with_len
 		cnt++;
 	}
 	*to = 0;
-	return (cnt);
+	return cnt;
 }
 
 static int
@@ -300,11 +319,11 @@ pg_euctw_mblen(const unsigned char *s)
 		len = 4;
 	else if (*s == SS3)
 		len = 3;
-	else if (*s & 0x80)
+	else if (IS_HIGHBIT_SET(*s))
 		len = 2;
 	else
 		len = 1;
-	return (len);
+	return len;
 }
 
 static int
@@ -316,11 +335,11 @@ pg_euctw_dsplen(const unsigned char *s)
 		len = 2;
 	else if (*s == SS3)
 		len = 2;
-	else if (*s & 0x80)
+	else if (IS_HIGHBIT_SET(*s))
 		len = 2;
 	else
-		len = 1;
-	return (len);
+		len = pg_ascii_dsplen(s);
+	return len;
 }
 
 /*
@@ -329,76 +348,62 @@ pg_euctw_dsplen(const unsigned char *s)
 static int
 pg_johab2wchar_with_len(const unsigned char *from, pg_wchar *to, int len)
 {
-	return (pg_euc2wchar_with_len(from, to, len));
+	return pg_euc2wchar_with_len(from, to, len);
 }
 
 static int
 pg_johab_mblen(const unsigned char *s)
 {
-	return (pg_euc_mblen(s));
+	return pg_euc_mblen(s);
 }
 
 static int
 pg_johab_dsplen(const unsigned char *s)
 {
-	return (pg_euc_dsplen(s));
+	return pg_euc_dsplen(s);
 }
 
 /*
- * convert UTF8 string to pg_wchar (UCS-4)
- * caller must allocate enough space for "to", including a trailing zero!
+ * convert UTF8 string to pg_wchar (UCS-2)
+ * caller should allocate enough space for "to"
  * len: length of from.
  * "from" not necessarily null terminated.
  */
 static int
 pg_utf2wchar_with_len(const unsigned char *from, pg_wchar *to, int len)
 {
-	int			cnt = 0;
-	uint32		c1,
+	unsigned char c1,
 				c2,
-				c3,
-				c4;
+				c3;
+	int			cnt = 0;
 
 	while (len > 0 && *from)
 	{
-		if ((*from & 0x80) == 0)
+		if (!IS_HIGHBIT_SET(*from))
 		{
 			*to = *from++;
 			len--;
 		}
-		else if ((*from & 0xe0) == 0xc0)
+		else if ((*from & 0xe0) == 0xc0 && len >= 2)
 		{
-			if (len < 2)
-				break;			/* drop trailing incomplete char */
 			c1 = *from++ & 0x1f;
 			c2 = *from++ & 0x3f;
-			*to = (c1 << 6) | c2;
+			*to = c1 << 6;
+			*to |= c2;
 			len -= 2;
 		}
-		else if ((*from & 0xf0) == 0xe0)
+		else if ((*from & 0xe0) == 0xe0 && len >= 3)
 		{
-			if (len < 3)
-				break;			/* drop trailing incomplete char */
 			c1 = *from++ & 0x0f;
 			c2 = *from++ & 0x3f;
 			c3 = *from++ & 0x3f;
-			*to = (c1 << 12) | (c2 << 6) | c3;
+			*to = c1 << 12;
+			*to |= c2 << 6;
+			*to |= c3;
 			len -= 3;
 		}
-		else if ((*from & 0xf8) == 0xf0)
-		{
-			if (len < 4)
-				break;			/* drop trailing incomplete char */
-			c1 = *from++ & 0x07;
-			c2 = *from++ & 0x3f;
-			c3 = *from++ & 0x3f;
-			c4 = *from++ & 0x3f;
-			*to = (c1 << 18) | (c2 << 12) | (c3 << 6) | c4;
-			len -= 4;
-		}
 		else
 		{
-			/* treat a bogus char as length 1; not ours to raise error */
 			*to = *from++;
 			len--;
 		}
@@ -406,24 +411,16 @@ pg_utf2wchar_with_len(const unsigned char *from, pg_wchar *to, int len)
 		cnt++;
 	}
 	*to = 0;
-	return (cnt);
+	return cnt;
 }
 
 /*
- * Return the byte length of a UTF8 character pointed to by s
- *
- * Note: in the current implementation we do not support UTF8 sequences
- * of more than 4 bytes; hence do NOT return a value larger than 4.
- * We return "1" for any leading byte that is either flat-out illegal or
- * indicates a length larger than we support.
- *
- * pg_utf2wchar_with_len(), utf2ucs(), pg_utf8_islegal(), and perhaps
- * other places would need to be fixed to change this.
+ * returns the byte length of a UTF8 character pointed to by s
  */
 int
 pg_utf_mblen(const unsigned char *s)
 {
-	int			len;
+	int			len = 1;
 
 	if ((*s & 0x80) == 0)
 		len = 1;
@@ -433,21 +430,186 @@ pg_utf_mblen(const unsigned char *s)
 		len = 3;
 	else if ((*s & 0xf8) == 0xf0)
 		len = 4;
-#ifdef NOT_USED
 	else if ((*s & 0xfc) == 0xf8)
 		len = 5;
 	else if ((*s & 0xfe) == 0xfc)
 		len = 6;
-#endif
-	else
-		len = 1;
 	return len;
 }
 
+/*
+ * This is an implementation of wcwidth() and wcswidth() as defined in
+ * "The Single UNIX Specification, Version 2, The Open Group, 1997"
+ * <http://www.UNIX-systems.org/online.html>
+ *
+ * Markus Kuhn -- 2001-09-08 -- public domain
+ *
+ * customised for PostgreSQL
+ *
+ * original available at : http://www.cl.cam.ac.uk/~mgk25/ucs/wcwidth.c
+ */
+
+struct mbinterval
+{
+	unsigned short first;
+	unsigned short last;
+};
+
+/* auxiliary function for binary search in interval table */
+static int
+mbbisearch(pg_wchar ucs, const struct mbinterval * table, int max)
+{
+	int			min = 0;
+	int			mid;
+
+	if (ucs < table[0].first || ucs > table[max].last)
+		return 0;
+	while (max >= min)
+	{
+		mid = (min + max) / 2;
+		if (ucs > table[mid].last)
+			min = mid + 1;
+		else if (ucs < table[mid].first)
+			max = mid - 1;
+		else
+			return 1;
+	}
+
+	return 0;
+}
+
+
+/* The following functions define the column width of an ISO 10646
+ * character as follows:
+ *
+ *	  - The null character (U+0000) has a column width of 0.
+ *
+ *	  - Other C0/C1 control characters and DEL will lead to a return
+ *		value of -1.
+ *
+ *	  - Non-spacing and enclosing combining characters (general
+ *		category code Mn or Me in the Unicode database) have a
+ *		column width of 0.
+ *
+ *	  - Other format characters (general category code Cf in the Unicode
+ *		database) and ZERO WIDTH SPACE (U+200B) have a column width of 0.
+ *
+ *	  - Hangul Jamo medial vowels and final consonants (U+1160-U+11FF)
+ *		have a column width of 0.
+ *
+ *	  - Spacing characters in the East Asian Wide (W) or East Asian
+ *		FullWidth (F) category as defined in Unicode Technical
+ *		Report #11 have a column width of 2.
+ *
+ *	  - All remaining characters (including all printable
+ *		ISO 8859-1 and WGL4 characters, Unicode control characters,
+ *		etc.) have a column width of 1.
+ *
+ * This implementation assumes that wchar_t characters are encoded
+ * in ISO 10646.
+ */
+
+static int
+ucs_wcwidth(pg_wchar ucs)
+{
+	/* sorted list of non-overlapping intervals of non-spacing characters */
+	static const struct mbinterval combining[] = {
+		{0x0300, 0x034E}, {0x0360, 0x0362}, {0x0483, 0x0486},
+		{0x0488, 0x0489}, {0x0591, 0x05A1}, {0x05A3, 0x05B9},
+		{0x05BB, 0x05BD}, {0x05BF, 0x05BF}, {0x05C1, 0x05C2},
+		{0x05C4, 0x05C4}, {0x064B, 0x0655}, {0x0670, 0x0670},
+		{0x06D6, 0x06E4}, {0x06E7, 0x06E8}, {0x06EA, 0x06ED},
+		{0x070F, 0x070F}, {0x0711, 0x0711}, {0x0730, 0x074A},
+		{0x07A6, 0x07B0}, {0x0901, 0x0902}, {0x093C, 0x093C},
+		{0x0941, 0x0948}, {0x094D, 0x094D}, {0x0951, 0x0954},
+		{0x0962, 0x0963}, {0x0981, 0x0981}, {0x09BC, 0x09BC},
+		{0x09C1, 0x09C4}, {0x09CD, 0x09CD}, {0x09E2, 0x09E3},
+		{0x0A02, 0x0A02}, {0x0A3C, 0x0A3C}, {0x0A41, 0x0A42},
+		{0x0A47, 0x0A48}, {0x0A4B, 0x0A4D}, {0x0A70, 0x0A71},
+		{0x0A81, 0x0A82}, {0x0ABC, 0x0ABC}, {0x0AC1, 0x0AC5},
+		{0x0AC7, 0x0AC8}, {0x0ACD, 0x0ACD}, {0x0B01, 0x0B01},
+		{0x0B3C, 0x0B3C}, {0x0B3F, 0x0B3F}, {0x0B41, 0x0B43},
+		{0x0B4D, 0x0B4D}, {0x0B56, 0x0B56}, {0x0B82, 0x0B82},
+		{0x0BC0, 0x0BC0}, {0x0BCD, 0x0BCD}, {0x0C3E, 0x0C40},
+		{0x0C46, 0x0C48}, {0x0C4A, 0x0C4D}, {0x0C55, 0x0C56},
+		{0x0CBF, 0x0CBF}, {0x0CC6, 0x0CC6}, {0x0CCC, 0x0CCD},
+		{0x0D41, 0x0D43}, {0x0D4D, 0x0D4D}, {0x0DCA, 0x0DCA},
+		{0x0DD2, 0x0DD4}, {0x0DD6, 0x0DD6}, {0x0E31, 0x0E31},
+		{0x0E34, 0x0E3A}, {0x0E47, 0x0E4E}, {0x0EB1, 0x0EB1},
+		{0x0EB4, 0x0EB9}, {0x0EBB, 0x0EBC}, {0x0EC8, 0x0ECD},
+		{0x0F18, 0x0F19}, {0x0F35, 0x0F35}, {0x0F37, 0x0F37},
+		{0x0F39, 0x0F39}, {0x0F71, 0x0F7E}, {0x0F80, 0x0F84},
+		{0x0F86, 0x0F87}, {0x0F90, 0x0F97}, {0x0F99, 0x0FBC},
+		{0x0FC6, 0x0FC6}, {0x102D, 0x1030}, {0x1032, 0x1032},
+		{0x1036, 0x1037}, {0x1039, 0x1039}, {0x1058, 0x1059},
+		{0x1160, 0x11FF}, {0x17B7, 0x17BD}, {0x17C6, 0x17C6},
+		{0x17C9, 0x17D3}, {0x180B, 0x180E}, {0x18A9, 0x18A9},
+		{0x200B, 0x200F}, {0x202A, 0x202E}, {0x206A, 0x206F},
+		{0x20D0, 0x20E3}, {0x302A, 0x302F}, {0x3099, 0x309A},
+		{0xFB1E, 0xFB1E}, {0xFE20, 0xFE23}, {0xFEFF, 0xFEFF},
+		{0xFFF9, 0xFFFB}
+	};
+
+	/* test for 8-bit control characters */
+	if (ucs == 0)
+		return 0;
+
+	if (ucs < 0x20 || (ucs >= 0x7f && ucs < 0xa0) || ucs > 0x0010ffff)
+		return -1;
+
+	/* binary search in table of non-spacing characters */
+	if (mbbisearch(ucs, combining,
+				   sizeof(combining) / sizeof(struct mbinterval) - 1))
+		return 0;
+
+	/*
+	 * if we arrive here, ucs is not a combining or C0/C1 control character
+	 */
+
+	return 1 +
+		(ucs >= 0x1100 &&
+		 (ucs <= 0x115f ||		/* Hangul Jamo init. consonants */
+		  (ucs >= 0x2e80 && ucs <= 0xa4cf && (ucs & ~0x0011) != 0x300a &&
+		   ucs != 0x303f) ||	/* CJK ... Yi */
+		  (ucs >= 0xac00 && ucs <= 0xd7a3) ||	/* Hangul Syllables */
+		  (ucs >= 0xf900 && ucs <= 0xfaff) ||	/* CJK Compatibility
+												 * Ideographs */
+		  (ucs >= 0xfe30 && ucs <= 0xfe6f) ||	/* CJK Compatibility Forms */
+		  (ucs >= 0xff00 && ucs <= 0xff5f) ||	/* Fullwidth Forms */
+		  (ucs >= 0xffe0 && ucs <= 0xffe6) ||
+		  (ucs >= 0x20000 && ucs <= 0x2ffff)));
+}
+
+static pg_wchar
+utf2ucs(const unsigned char *c)
+{
+	/*
+	 * one char version of pg_utf2wchar_with_len. no control here, c must
+	 * point to a large enough string
+	 */
+	if ((*c & 0x80) == 0)
+		return (pg_wchar) c[0];
+	else if ((*c & 0xe0) == 0xc0)
+		return (pg_wchar) (((c[0] & 0x1f) << 6) |
+						   (c[1] & 0x3f));
+	else if ((*c & 0xf0) == 0xe0)
+		return (pg_wchar) (((c[0] & 0x0f) << 12) |
+						   ((c[1] & 0x3f) << 6) |
+						   (c[2] & 0x3f));
+	else if ((*c & 0xf0) == 0xf0)
+		return (pg_wchar) (((c[0] & 0x07) << 18) |
+						   ((c[1] & 0x3f) << 12) |
+						   ((c[2] & 0x3f) << 6) |
+						   (c[3] & 0x3f));
+	else
+		/* that is an invalid code on purpose */
+		return 0xffffffff;
+}
+
 static int
 pg_utf_dsplen(const unsigned char *s)
 {
-	return 1;					/* XXX fix me! */
+	return ucs_wcwidth(utf2ucs(s));
 }
 
 /*
@@ -500,7 +662,7 @@ pg_mule2wchar_with_len(const unsigned char *from, pg_wchar *to, int len)
 		cnt++;
 	}
 	*to = 0;
-	return (cnt);
+	return cnt;
 }
 
 int
@@ -517,16 +679,27 @@ pg_mule_mblen(const unsigned char *s)
 	else if (IS_LCPRV2(*s))
 		len = 4;
 	else
-	{							/* assume ASCII */
-		len = 1;
-	}
-	return (len);
+		len = 1;				/* assume ASCII */
+	return len;
 }
 
 static int
 pg_mule_dsplen(const unsigned char *s)
 {
-	return 1;					/* XXX fix me! */
+	int			len;
+
+	if (IS_LC1(*s))
+		len = 1;
+	else if (IS_LCPRV1(*s))
+		len = 1;
+	else if (IS_LC2(*s))
+		len = 2;
+	else if (IS_LCPRV2(*s))
+		len = 2;
+	else
+		len = 1;				/* assume ASCII */
+
+	return len;
 }
 
 /*
@@ -544,19 +717,19 @@ pg_latin12wchar_with_len(const unsigned char *from, pg_wchar *to, int len)
 		cnt++;
 	}
 	*to = 0;
-	return (cnt);
+	return cnt;
 }
 
 static int
 pg_latin1_mblen(const unsigned char *s)
 {
-	return (1);
+	return 1;
 }
 
 static int
 pg_latin1_dsplen(const unsigned char *s)
 {
-	return (1);
+	return pg_ascii_dsplen(s);
 }
 
 /*
@@ -568,18 +741,12 @@ pg_sjis_mblen(const unsigned char *s)
 	int			len;
 
 	if (*s >= 0xa1 && *s <= 0xdf)
-	{							/* 1 byte kana? */
-		len = 1;
-	}
-	else if (*s > 0x7f)
-	{							/* kanji? */
-		len = 2;
-	}
+		len = 1;				/* 1 byte kana? */
+	else if (IS_HIGHBIT_SET(*s))
+		len = 2;				/* kanji? */
 	else
-	{							/* should be ASCII */
-		len = 1;
-	}
-	return (len);
+		len = 1;				/* should be ASCII */
+	return len;
 }
 
 static int
@@ -588,18 +755,12 @@ pg_sjis_dsplen(const unsigned char *s)
 	int			len;
 
 	if (*s >= 0xa1 && *s <= 0xdf)
-	{							/* 1 byte kana? */
-		len = 1;
-	}
-	else if (*s > 0x7f)
-	{							/* kanji? */
-		len = 2;
-	}
+		len = 1;				/* 1 byte kana? */
+	else if (IS_HIGHBIT_SET(*s))
+		len = 2;				/* kanji? */
 	else
-	{							/* should be ASCII */
-		len = 1;
-	}
-	return (len);
+		len = pg_ascii_dsplen(s);		/* should be ASCII */
+	return len;
 }
 
 /*
@@ -610,15 +771,11 @@ pg_big5_mblen(const unsigned char *s)
 {
 	int			len;
 
-	if (*s > 0x7f)
-	{							/* kanji? */
-		len = 2;
-	}
+	if (IS_HIGHBIT_SET(*s))
+		len = 2;				/* kanji? */
 	else
-	{							/* should be ASCII */
-		len = 1;
-	}
-	return (len);
+		len = 1;				/* should be ASCII */
+	return len;
 }
 
 static int
@@ -626,15 +783,11 @@ pg_big5_dsplen(const unsigned char *s)
 {
 	int			len;
 
-	if (*s > 0x7f)
-	{							/* kanji? */
-		len = 2;
-	}
+	if (IS_HIGHBIT_SET(*s))
+		len = 2;				/* kanji? */
 	else
-	{							/* should be ASCII */
-		len = 1;
-	}
-	return (len);
+		len = pg_ascii_dsplen(s);		/* should be ASCII */
+	return len;
 }
 
 /*
@@ -645,15 +798,11 @@ pg_gbk_mblen(const unsigned char *s)
 {
 	int			len;
 
-	if (*s > 0x7f)
-	{							/* kanji? */
-		len = 2;
-	}
+	if (IS_HIGHBIT_SET(*s))
+		len = 2;				/* kanji? */
 	else
-	{							/* should be ASCII */
-		len = 1;
-	}
-	return (len);
+		len = 1;				/* should be ASCII */
+	return len;
 }
 
 static int
@@ -661,15 +810,11 @@ pg_gbk_dsplen(const unsigned char *s)
 {
 	int			len;
 
-	if (*s > 0x7f)
-	{							/* kanji? */
-		len = 2;
-	}
+	if (IS_HIGHBIT_SET(*s))
+		len = 2;				/* kanji? */
 	else
-	{							/* should be ASCII */
-		len = 1;
-	}
-	return (len);
+		len = pg_ascii_dsplen(s);		/* should be ASCII */
+	return len;
 }
 
 /*
@@ -680,15 +825,11 @@ pg_uhc_mblen(const unsigned char *s)
 {
 	int			len;
 
-	if (*s > 0x7f)
-	{							/* 2byte? */
-		len = 2;
-	}
+	if (IS_HIGHBIT_SET(*s))
+		len = 2;				/* 2byte? */
 	else
-	{							/* should be ASCII */
-		len = 1;
-	}
-	return (len);
+		len = 1;				/* should be ASCII */
+	return len;
 }
 
 static int
@@ -696,15 +837,11 @@ pg_uhc_dsplen(const unsigned char *s)
 {
 	int			len;
 
-	if (*s > 0x7f)
-	{							/* 2byte? */
-		len = 2;
-	}
+	if (IS_HIGHBIT_SET(*s))
+		len = 2;				/* 2byte? */
 	else
-	{							/* should be ASCII */
-		len = 1;
-	}
-	return (len);
+		len = pg_ascii_dsplen(s);		/* should be ASCII */
+	return len;
 }
 
 /*
@@ -716,10 +853,8 @@ pg_gb18030_mblen(const unsigned char *s)
 {
 	int			len;
 
-	if (*s <= 0x7f)
-	{							/* ASCII */
-		len = 1;
-	}
+	if (!IS_HIGHBIT_SET(*s))
+		len = 1;				/* ASCII */
 	else
 	{
 		if ((*(s + 1) >= 0x40 && *(s + 1) <= 0x7e) || (*(s + 1) >= 0x80 && *(s + 1) <= 0xfe))
@@ -729,7 +864,7 @@ pg_gb18030_mblen(const unsigned char *s)
 		else
 			len = 2;
 	}
-	return (len);
+	return len;
 }
 
 static int
@@ -737,13 +872,11 @@ pg_gb18030_dsplen(const unsigned char *s)
 {
 	int			len;
 
-	if (*s <= 0x7f)
-	{							/* ASCII */
-		len = 1;
-	}
-	else
+	if (IS_HIGHBIT_SET(*s))
 		len = 2;
-	return (len);
+	else
+		len = pg_ascii_dsplen(s);		/* ASCII */
+	return len;
 }
 
 /*
@@ -775,13 +908,14 @@ static int
 pg_eucjp_verifier(const unsigned char *s, int len)
 {
 	int			l;
-	unsigned char c1, c2;
+	unsigned char c1,
+				c2;
 
 	c1 = *s++;
 
 	switch (c1)
 	{
-		case SS2:		/* JIS X 0201 */
+		case SS2:				/* JIS X 0201 */
 			l = 2;
 			if (l > len)
 				return -1;
@@ -790,7 +924,7 @@ pg_eucjp_verifier(const unsigned char *s, int len)
 				return -1;
 			break;
 
-		case SS3:		/* JIS X 0212 */
+		case SS3:				/* JIS X 0212 */
 			l = 3;
 			if (l > len)
 				return -1;
@@ -814,7 +948,8 @@ pg_eucjp_verifier(const unsigned char *s, int len)
 				if (!IS_EUC_RANGE_VALID(c2))
 					return -1;
 			}
-			else		/* must be ASCII */
+			else
+				/* must be ASCII */
 			{
 				l = 1;
 			}
@@ -828,7 +963,8 @@ static int
 pg_euckr_verifier(const unsigned char *s, int len)
 {
 	int			l;
-	unsigned char c1, c2;
+	unsigned char c1,
+				c2;
 
 	c1 = *s++;
 
@@ -843,7 +979,8 @@ pg_euckr_verifier(const unsigned char *s, int len)
 		if (!IS_EUC_RANGE_VALID(c2))
 			return -1;
 	}
-	else		/* must be ASCII */
+	else
+		/* must be ASCII */
 	{
 		l = 1;
 	}
@@ -858,13 +995,14 @@ static int
 pg_euctw_verifier(const unsigned char *s, int len)
 {
 	int			l;
-	unsigned char c1, c2;
+	unsigned char c1,
+				c2;
 
 	c1 = *s++;
 
 	switch (c1)
 	{
-		case SS2:		/* CNS 11643 Plane 1-7 */
+		case SS2:				/* CNS 11643 Plane 1-7 */
 			l = 4;
 			if (l > len)
 				return -1;
@@ -879,7 +1017,7 @@ pg_euctw_verifier(const unsigned char *s, int len)
 				return -1;
 			break;
 
-		case SS3:		/* unused */
+		case SS3:				/* unused */
 			return -1;
 
 		default:
@@ -893,7 +1031,8 @@ pg_euctw_verifier(const unsigned char *s, int len)
 				if (!IS_EUC_RANGE_VALID(c2))
 					return -1;
 			}
-			else		/* must be ASCII */
+			else
+				/* must be ASCII */
 			{
 				l = 1;
 			}
@@ -905,7 +1044,8 @@ pg_euctw_verifier(const unsigned char *s, int len)
 static int
 pg_johab_verifier(const unsigned char *s, int len)
 {
-	int l, mbl;
+	int			l,
+				mbl;
 	unsigned char c;
 
 	l = mbl = pg_johab_mblen(s);
@@ -928,7 +1068,8 @@ pg_johab_verifier(const unsigned char *s, int len)
 static int
 pg_mule_verifier(const unsigned char *s, int len)
 {
-	int l, mbl;
+	int			l,
+				mbl;
 	unsigned char c;
 
 	l = mbl = pg_mule_mblen(s);
@@ -954,8 +1095,10 @@ pg_latin1_verifier(const unsigned char *s, int len)
 static int
 pg_sjis_verifier(const unsigned char *s, int len)
 {
-	int l, mbl;
-	unsigned char c1, c2;
+	int			l,
+				mbl;
+	unsigned char c1,
+				c2;
 
 	l = mbl = pg_sjis_mblen(s);
 
@@ -975,7 +1118,8 @@ pg_sjis_verifier(const unsigned char *s, int len)
 static int
 pg_big5_verifier(const unsigned char *s, int len)
 {
-	int l, mbl;
+	int			l,
+				mbl;
 
 	l = mbl = pg_big5_mblen(s);
 
@@ -994,7 +1138,8 @@ pg_big5_verifier(const unsigned char *s, int len)
 static int
 pg_gbk_verifier(const unsigned char *s, int len)
 {
-	int l, mbl;
+	int			l,
+				mbl;
 
 	l = mbl = pg_gbk_mblen(s);
 
@@ -1013,7 +1158,8 @@ pg_gbk_verifier(const unsigned char *s, int len)
 static int
 pg_uhc_verifier(const unsigned char *s, int len)
 {
-	int l, mbl;
+	int			l,
+				mbl;
 
 	l = mbl = pg_uhc_mblen(s);
 
@@ -1032,7 +1178,8 @@ pg_uhc_verifier(const unsigned char *s, int len)
 static int
 pg_gb18030_verifier(const unsigned char *s, int len)
 {
-	int l, mbl;
+	int			l,
+				mbl;
 
 	l = mbl = pg_gb18030_mblen(s);
 
@@ -1051,7 +1198,7 @@ pg_gb18030_verifier(const unsigned char *s, int len)
 static int
 pg_utf8_verifier(const unsigned char *s, int len)
 {
-	int l = pg_utf_mblen(s);
+	int			l = pg_utf_mblen(s);
 
 	if (len < l)
 		return -1;
@@ -1139,41 +1286,45 @@ pg_utf8_islegal(const unsigned char *source, int length)
  *-------------------------------------------------------------------
  */
 pg_wchar_tbl pg_wchar_table[] = {
-	{pg_ascii2wchar_with_len, pg_ascii_mblen, pg_ascii_dsplen, pg_ascii_verifier, 1},		/* 0; PG_SQL_ASCII	*/
-	{pg_eucjp2wchar_with_len, pg_eucjp_mblen, pg_eucjp_dsplen, pg_eucjp_verifier, 3},		/* 1; PG_EUC_JP */
-	{pg_euccn2wchar_with_len, pg_euccn_mblen, pg_euccn_dsplen, pg_euccn_verifier, 2},		/* 2; PG_EUC_CN */
-	{pg_euckr2wchar_with_len, pg_euckr_mblen, pg_euckr_dsplen, pg_euckr_verifier, 3},		/* 3; PG_EUC_KR */
-	{pg_euctw2wchar_with_len, pg_euctw_mblen, pg_euctw_dsplen, pg_euctw_verifier, 4},		/* 4; PG_EUC_TW */
-	{pg_johab2wchar_with_len, pg_johab_mblen, pg_johab_dsplen, pg_johab_verifier, 3},		/* 5; PG_JOHAB */
+	{pg_ascii2wchar_with_len, pg_ascii_mblen, pg_ascii_dsplen, pg_ascii_verifier, 1},	/* 0; PG_SQL_ASCII	*/
+	{pg_eucjp2wchar_with_len, pg_eucjp_mblen, pg_eucjp_dsplen, pg_eucjp_verifier, 3},	/* 1; PG_EUC_JP */
+	{pg_euccn2wchar_with_len, pg_euccn_mblen, pg_euccn_dsplen, pg_euccn_verifier, 3},	/* 2; PG_EUC_CN */
+	{pg_euckr2wchar_with_len, pg_euckr_mblen, pg_euckr_dsplen, pg_euckr_verifier, 3},	/* 3; PG_EUC_KR */
+	{pg_euctw2wchar_with_len, pg_euctw_mblen, pg_euctw_dsplen, pg_euctw_verifier, 3},	/* 4; PG_EUC_TW */
+	{pg_johab2wchar_with_len, pg_johab_mblen, pg_johab_dsplen, pg_johab_verifier, 3},	/* 5; PG_JOHAB */
 	{pg_utf2wchar_with_len, pg_utf_mblen, pg_utf_dsplen, pg_utf8_verifier, 4},	/* 6; PG_UTF8 */
-	{pg_mule2wchar_with_len, pg_mule_mblen, pg_mule_dsplen, pg_mule_verifier, 4}, /* 7; PG_MULE_INTERNAL */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 8; PG_LATIN1 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 9; PG_LATIN2 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 10; PG_LATIN3 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 11; PG_LATIN4 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 12; PG_LATIN5 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 13; PG_LATIN6 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 14; PG_LATIN7 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 15; PG_LATIN8 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 16; PG_LATIN9 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 17; PG_LATIN10 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 18; PG_WIN1256 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 19; PG_WIN1258 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 20; PG_WIN874 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 21; PG_KOI8 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 22; PG_WIN1251 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 22; PG_WIN1252 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 23; PG_WIN866 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 24; ISO-8859-5 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 25; ISO-8859-6 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 26; ISO-8859-7 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 27; ISO-8859-8 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},	/* 28; PG_WIN1250 */
-	{0, pg_sjis_mblen, pg_sjis_dsplen, pg_sjis_verifier, 2},		/* 29; PG_SJIS */
-	{0, pg_big5_mblen, pg_big5_dsplen, pg_big5_verifier, 2},		/* 30; PG_BIG5 */
-	{0, pg_gbk_mblen, pg_gbk_dsplen, pg_gbk_verifier, 2},		/* 31; PG_GBK */
-	{0, pg_uhc_mblen, pg_uhc_dsplen, pg_uhc_verifier, 2},		/* 32; PG_UHC */
-	{0, pg_gb18030_mblen, pg_gb18030_dsplen, pg_gb18030_verifier, 4} /* 33; PG_GB18030 */
+	{pg_mule2wchar_with_len, pg_mule_mblen, pg_mule_dsplen, pg_mule_verifier, 3},		/* 7; PG_MULE_INTERNAL */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 8; PG_LATIN1 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 9; PG_LATIN2 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 10; PG_LATIN3 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 11; PG_LATIN4 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 12; PG_LATIN5 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 13; PG_LATIN6 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 14; PG_LATIN7 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 15; PG_LATIN8 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 16; PG_LATIN9 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 17; PG_LATIN10 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 18; PG_WIN1256 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 19; PG_WIN1258 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 20; PG_WIN874 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 21; PG_KOI8 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 22; PG_WIN1251 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 22; PG_WIN1252 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 23; PG_WIN866 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 24; ISO-8859-5 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 25; ISO-8859-6 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 26; ISO-8859-7 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 27; ISO-8859-8 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 28; PG_WIN1250 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 29; PG_WIN1253 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 30; PG_WIN1254 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 31; PG_WIN1255 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, pg_latin1_verifier, 1},		/* 32; PG_WIN1257 */
+	{0, pg_sjis_mblen, pg_sjis_dsplen, pg_sjis_verifier, 2},	/* 33; PG_SJIS */
+	{0, pg_big5_mblen, pg_big5_dsplen, pg_big5_verifier, 2},	/* 34; PG_BIG5 */
+	{0, pg_gbk_mblen, pg_gbk_dsplen, pg_gbk_verifier, 2},		/* 35; PG_GBK */
+	{0, pg_uhc_mblen, pg_uhc_dsplen, pg_uhc_verifier, 2},		/* 36; PG_UHC */
+	{0, pg_gb18030_mblen, pg_gb18030_dsplen, pg_gb18030_verifier, 2}	/* 37; PG_GB18030 */
 };
 
 /* returns the byte length of a word for mule internal code */
@@ -1223,8 +1374,8 @@ pg_encoding_verifymb(int encoding, const char *mbstr, int len)
 
 	return ((encoding >= 0 &&
 			 encoding < sizeof(pg_wchar_table) / sizeof(pg_wchar_tbl)) ?
-		((*pg_wchar_table[encoding].mbverify) ((const unsigned char *) mbstr, len)) :
-	((*pg_wchar_table[PG_SQL_ASCII].mbverify) ((const unsigned char *) mbstr, len)));
+			((*pg_wchar_table[encoding].mbverify) ((const unsigned char *) mbstr, len)) :
+			((*pg_wchar_table[PG_SQL_ASCII].mbverify) ((const unsigned char *) mbstr, len)));
 }
 
 /*
@@ -1352,9 +1503,9 @@ report_invalid_encoding(int encoding, const char *mbstr, int len)
 			 errmsg("invalid byte sequence for encoding \"%s\": 0x%s",
 					pg_enc2name_tbl[encoding].name,
 					buf),
-			 errhint("This error can also happen if the byte sequence does not "
-			 		 "match the encoding expected by the server, which is controlled "
-					 "by \"client_encoding\".")));
+		  errhint("This error can also happen if the byte sequence does not "
+			"match the encoding expected by the server, which is controlled "
+				  "by \"client_encoding\".")));
 }
 
 /*
@@ -1381,10 +1532,10 @@ report_untranslatable_char(int src_encoding, int dest_encoding,
 
 	ereport(ERROR,
 			(errcode(ERRCODE_UNTRANSLATABLE_CHARACTER),
-			 errmsg("character 0x%s of encoding \"%s\" has no equivalent in \"%s\"",
-					buf,
-					pg_enc2name_tbl[src_encoding].name,
-					pg_enc2name_tbl[dest_encoding].name)));
+	  errmsg("character 0x%s of encoding \"%s\" has no equivalent in \"%s\"",
+			 buf,
+			 pg_enc2name_tbl[src_encoding].name,
+			 pg_enc2name_tbl[dest_encoding].name)));
 }
 
 #endif
